@@ -1,34 +1,42 @@
 # coding: utf-8
 # flake8: noqa
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import six
 
-if six.PY2:
-    from .tempfile import TemporaryDirectory
-    from .contextlib import ExitStack
-else:
-    from tempfile import TemporaryDirectory
-    from contextlib import ExitStack
-
 from .pip_compat import (
-    InstallRequirement,
-    parse_requirements,
-    RequirementSet,
-    user_cache_dir,
+    DEV_PKGS,
     FAVORITE_HASH,
-    is_file_url,
-    url_to_path,
-    PackageFinder,
+    PIP_VERSION,
     FormatControl,
+    InstallationCandidate,
+    InstallCommand,
+    InstallationError,
+    InstallRequirement,
+    Link,
+    PackageFinder,
+    PyPI,
+    RequirementSet,
+    RequirementTracker,
+    Resolver,
+    SafeFileCache,
+    VcsSupport,
     Wheel,
-    Command,
+    WheelCache,
     cmdoptions,
     get_installed_distributions,
-    PyPI,
-    SafeFileCache,
-    InstallationError,
-    install_req_from_line,
     install_req_from_editable,
+    install_req_from_line,
+    parse_requirements,
+    path_to_url,
+    pip_version,
+    stdlib_pkgs,
+    url_to_path,
+    user_cache_dir,
+    normalize_path,
 )
+
+if six.PY2:
+    from .tempfile import TemporaryDirectory
+else:
+    from tempfile import TemporaryDirectory
